package meta

import (
	"encoding/json"
	"git.loafle.net/overflow/commons_go/model/timestamp"
	"strconv"

)

const (
	INFRA_VENDOR_TYPE_SERIVCE = 7
	INFRA_VENDOR_START_IDX    = 38
	INFRA_VENDOR_MYSQL        = INFRA_VENDOR_START_IDX + 1
	INFRA_VENDOR_POSTGRESQL   = INFRA_VENDOR_START_IDX + 2
	INFRA_VENDOR_WMI          = INFRA_VENDOR_START_IDX + 3
	INFRA_VENDOR_SNMPV2       = INFRA_VENDOR_START_IDX + 4

	INFRA_VENDOR_UNKNOWN       = INFRA_VENDOR_START_IDX + 5
)

var metaInfraVendorMap = map[int]string{
	INFRA_VENDOR_MYSQL:      "MYSQL",
	INFRA_VENDOR_POSTGRESQL: "POSTGRESQL",
	INFRA_VENDOR_WMI:        "WMI",
	INFRA_VENDOR_SNMPV2:     "SNMPV2",
	INFRA_VENDOR_UNKNOWN:"UNKNOWN",
}

var metaInfraVendorMapStr map[string]int

func init() {
	metaInfraVendorMapStr = make(map[string]int)

	for k, v := range metaInfraVendorMap {
		metaInfraVendorMapStr[v] = k
	}
}

type MetaInfraVendor struct {
	Id            json.Number         `json:"id,Number,omitempty"`
	Name          string              `json:"name,omitempty"`
	CreateDate    timestamp.Timestamp `json:"createDate,omitempty"`
	MetaInfraType MetaInfraType       `json:"metaInfraType,omitempty"`
}

func NewMetaInfraVendorByService(serviceName string) *MetaInfraVendor {

	idx := metaInfraVendorMapStr[serviceName]

	if idx <= 0 {
		return &MetaInfraVendor{
			Id:            json.Number(strconv.Itoa(INFRA_VENDOR_UNKNOWN)),
			Name:          "UNKNOWN",
			MetaInfraType: MetaInfraType{Id: json.Number(strconv.Itoa(INFRA_VENDOR_TYPE_SERIVCE))},
		}
	}

	return &MetaInfraVendor{
		Id:            json.Number(strconv.Itoa(idx)),
		Name:          serviceName,
		MetaInfraType: MetaInfraType{Id: json.Number(strconv.Itoa(INFRA_VENDOR_TYPE_SERIVCE))},
	}
}
